Type: Package
Package: lubridate
Title: Make Dealing with Dates a Little Easier
Version: 1.9.5
Authors@R: c(
    person("Vitalie", "Spinu", , "spinuvit@gmail.com", role = c("aut", "cre")),
    person("Garrett", "Grolemund", role = "aut"),
    person("Hadley", "Wickham", role = "aut"),
    person("Davis", "Vaughan", role = "ctb"),
    person("Ian", "Lyttle", role = "ctb"),
    person("Imanuel", "Costigan", role = "ctb"),
    person("Jason", "Law", role = "ctb"),
    person("Doug", "Mitarotonda", role = "ctb"),
    person("Joseph", "Larmarange", role = "ctb"),
    person("Jonathan", "Boiser", role = "ctb"),
    person("Chel Hee", "Lee", role = "ctb")
  )
Maintainer: Vitalie Spinu <spinuvit@gmail.com>
Description: Functions to work with date-times and time-spans: fast and
    user friendly parsing of date-time data, extraction and updating of
    components of a date-time (years, months, days, hours, minutes, and
    seconds), algebraic manipulation on date-time and time-span objects.
    The 'lubridate' package has a consistent and memorable syntax that
    makes working with dates easy and fun.
License: MIT + file LICENSE
URL: https://lubridate.tidyverse.org,
        https://github.com/tidyverse/lubridate
BugReports: https://github.com/tidyverse/lubridate/issues
Depends: methods, R (>= 3.2)
Imports: generics, timechange (>= 0.4.0)
Suggests: covr, knitr, rmarkdown, testthat (>= 2.1.0), vctrs (>= 0.6.5)
Enhances: chron, data.table, timeDate, tis, zoo
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
SystemRequirements: A system with zoneinfo data (e.g.
        /usr/share/zoneinfo). On Windows the zoneinfo included with R
        is used.
Collate: 'Dates.r' 'POSIXt.r' 'util.r' 'parse.r' 'timespans.r'
        'intervals.r' 'difftimes.r' 'durations.r' 'periods.r'
        'accessors-date.R' 'accessors-day.r' 'accessors-dst.r'
        'accessors-hour.r' 'accessors-minute.r' 'accessors-month.r'
        'accessors-quarter.r' 'accessors-second.r' 'accessors-tz.r'
        'accessors-week.r' 'accessors-year.r' 'am-pm.r' 'time-zones.r'
        'numeric.r' 'coercion.r' 'constants.r' 'cyclic_encoding.r'
        'data.r' 'decimal-dates.r' 'deprecated.r' 'format_ISO8601.r'
        'guess.r' 'hidden.r' 'instants.r' 'leap-years.r'
        'ops-addition.r' 'ops-compare.r' 'ops-division.r'
        'ops-integer-division.r' 'ops-m+.r' 'ops-modulo.r'
        'ops-multiplication.r' 'ops-subtraction.r' 'package.r'
        'pretty.r' 'round.r' 'stamp.r' 'tzdir.R' 'update.r' 'vctrs.R'
        'zzz.R'
NeedsCompilation: yes
Packaged: 2026-02-03 09:30:08 UTC; vitalie
Author: Vitalie Spinu [aut, cre],
  Garrett Grolemund [aut],
  Hadley Wickham [aut],
  Davis Vaughan [ctb],
  Ian Lyttle [ctb],
  Imanuel Costigan [ctb],
  Jason Law [ctb],
  Doug Mitarotonda [ctb],
  Joseph Larmarange [ctb],
  Jonathan Boiser [ctb],
  Chel Hee Lee [ctb]
Repository: CRAN
Date/Publication: 2026-02-04 09:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 05:40:21 UTC; unix
Archs: lubridate.so.dSYM
