# Generated by roxygen2: do not edit by hand

S3method(as.POSIXct,fts)
S3method(as.POSIXct,irts)
S3method(as.POSIXct,timeSeries)
S3method(as.POSIXct,xts)
S3method(as.POSIXct,zoo)
S3method(as.POSIXlt,fts)
S3method(as.POSIXlt,irts)
S3method(as.POSIXlt,timeSeries)
S3method(as.POSIXlt,tis)
S3method(as.POSIXlt,xts)
S3method(as.POSIXlt,zoo)
S3method(as.list,Interval)
S3method(c,Duration)
S3method(c,Interval)
S3method(c,Period)
S3method(date,Period)
S3method(date,default)
S3method(decimal_date,default)
S3method(decimal_date,zoo)
S3method(dst,default)
S3method(force_tz,default)
S3method(format,Duration)
S3method(format,Interval)
S3method(format,Period)
S3method(hour,Period)
S3method(hour,default)
S3method(intersect,Interval)
S3method(mday,Period)
S3method(mday,default)
S3method(minute,Period)
S3method(minute,default)
S3method(month,Period)
S3method(month,default)
S3method(month,numeric)
S3method(months,numeric)
S3method(qday,default)
S3method(reclass_date,Date)
S3method(reclass_date,IDate)
S3method(reclass_date,ITime)
S3method(reclass_date,POSIXct)
S3method(reclass_date,POSIXlt)
S3method(reclass_date,chron)
S3method(reclass_date,ti)
S3method(reclass_date,timeDate)
S3method(second,Period)
S3method(second,default)
S3method(setdiff,Interval)
S3method(summary,Duration)
S3method(summary,Interval)
S3method(summary,Period)
S3method(tz,Date)
S3method(tz,POSIXt)
S3method(tz,character)
S3method(tz,default)
S3method(tz,logical)
S3method(tz,timeSeries)
S3method(tz,yearmon)
S3method(tz,yearqtr)
S3method(tz,zoo)
S3method(tz,zooreg)
S3method(union,Interval)
S3method(unique,Interval)
S3method(update,Date)
S3method(update,POSIXt)
S3method(wday,default)
S3method(wday,numeric)
S3method(with_tz,default)
S3method(xtfrm,Period)
S3method(yday,default)
S3method(year,Period)
S3method(year,default)
export("%--%")
export("%m+%")
export("%m-%")
export("%within%")
export("date<-")
export("day<-")
export("hour<-")
export("int_end<-")
export("int_start<-")
export("mday<-")
export("minute<-")
export("month<-")
export("qday<-")
export("second<-")
export("tz<-")
export("wday<-")
export("week<-")
export("yday<-")
export("year<-")
export(Date)
export(NA_Date_)
export(NA_POSIXct_)
export(POSIXct)
export(add_with_rollback)
export(am)
export(as.difftime)
export(as.duration)
export(as.interval)
export(as.period)
export(as_date)
export(as_datetime)
export(ceiling_date)
export(cyclic_encoding)
export(date)
export(date_decimal)
export(day)
export(days)
export(days_in_month)
export(ddays)
export(decimal_date)
export(dhours)
export(dmicroseconds)
export(dmilliseconds)
export(dminutes)
export(dmonths)
export(dmy)
export(dmy_h)
export(dmy_hm)
export(dmy_hms)
export(dnanoseconds)
export(dpicoseconds)
export(dseconds)
export(dst)
export(duration)
export(dweeks)
export(dyears)
export(dym)
export(epiweek)
export(epiyear)
export(fast_strptime)
export(fit_to_timeline)
export(floor_date)
export(force_tz)
export(force_tzs)
export(format_ISO8601)
export(guess_formats)
export(hm)
export(hms)
export(hour)
export(hours)
export(int_aligns)
export(int_diff)
export(int_end)
export(int_flip)
export(int_length)
export(int_overlaps)
export(int_shift)
export(int_standardize)
export(int_start)
export(intersect)
export(interval)
export(is.Date)
export(is.POSIXct)
export(is.POSIXlt)
export(is.POSIXt)
export(is.difftime)
export(is.duration)
export(is.instant)
export(is.interval)
export(is.period)
export(is.timepoint)
export(is.timespan)
export(isoweek)
export(isoyear)
export(leap_year)
export(local_time)
export(make_date)
export(make_datetime)
export(make_difftime)
export(mday)
export(mdy)
export(mdy_h)
export(mdy_hm)
export(mdy_hms)
export(microseconds)
export(milliseconds)
export(minute)
export(minutes)
export(month)
export(ms)
export(my)
export(myd)
export(nanoseconds)
export(now)
export(origin)
export(parse_date_time)
export(parse_date_time2)
export(period)
export(period_to_seconds)
export(picoseconds)
export(pm)
export(pretty_dates)
export(qday)
export(quarter)
export(reclass_date)
export(reclass_timespan)
export(rollback)
export(rollbackward)
export(rollforward)
export(round_date)
export(second)
export(seconds)
export(seconds_to_period)
export(semester)
export(setdiff)
export(stamp)
export(stamp_date)
export(stamp_time)
export(time_length)
export(today)
export(tz)
export(union)
export(wday)
export(week)
export(weeks)
export(with_tz)
export(yday)
export(ydm)
export(ydm_h)
export(ydm_hm)
export(ydm_hms)
export(year)
export(years)
export(ym)
export(ymd)
export(ymd_h)
export(ymd_hm)
export(ymd_hms)
export(yq)
exportClasses(Duration)
exportClasses(Interval)
exportClasses(Period)
exportClasses(Timespan)
exportMethods("$")
exportMethods("$<-")
exportMethods("%%")
exportMethods("%/%")
exportMethods("%m+%")
exportMethods("%m-%")
exportMethods("*")
exportMethods("+")
exportMethods("-")
exportMethods("/")
exportMethods("[")
exportMethods("[<-")
exportMethods("[[")
exportMethods("[[<-")
exportMethods("date<-")
exportMethods("day<-")
exportMethods("hour<-")
exportMethods("minute<-")
exportMethods("month<-")
exportMethods("second<-")
exportMethods("year<-")
exportMethods(Arith)
exportMethods(Compare)
exportMethods(as.character)
exportMethods(as.difftime)
exportMethods(as.integer)
exportMethods(as.interval)
exportMethods(as.numeric)
exportMethods(as_date)
exportMethods(as_datetime)
exportMethods(format_ISO8601)
exportMethods(reclass_timespan)
exportMethods(rep)
exportMethods(show)
importFrom(generics,as.difftime)
importFrom(generics,intersect)
importFrom(generics,setdiff)
importFrom(generics,union)
importFrom(methods,"coerce<-")
importFrom(methods,"slot<-")
importFrom(methods,Arith)
importFrom(methods,Compare)
importFrom(methods,allNames)
importFrom(methods,callGeneric)
importFrom(methods,initialize)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(methods,validObject)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(timechange,time_add)
importFrom(timechange,time_force_tz)
importFrom(timechange,time_get)
importFrom(timechange,time_update)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
useDynLib(lubridate, .registration=TRUE)
