# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

C_local_tz <- function() {
    .Call(`_lubridate_C_local_tz`)
}

C_valid_tz <- function(tz_name) {
    .Call(`_lubridate_C_valid_tz`, tz_name)
}

C_update_dt <- function(dt, year, month, yday, mday, wday, hour, minute, second, tz = NULL, roll = FALSE, week_start = 7L) {
    .Call(`_lubridate_C_update_dt`, dt, year, month, yday, mday, wday, hour, minute, second, tz, roll, week_start)
}

C_force_tz <- function(dt, tz, roll = FALSE) {
    .Call(`_lubridate_C_force_tz`, dt, tz, roll)
}

C_force_tzs <- function(dt, tzs, tz_out, roll = FALSE) {
    .Call(`_lubridate_C_force_tzs`, dt, tzs, tz_out, roll)
}

C_local_time <- function(dt, tzs) {
    .Call(`_lubridate_C_local_time`, dt, tzs)
}

